<?php
/**
 * @version 2.0.1
 * @package moseasymedia
 * @copyright Copyright &copy; 2008, Brilaps LLC
 * @author Ozgur Cem Sen
 * @email code@brilaps.com
 * @link http://brilaps.com || http://wiki.brilaps.com
 * @license http://www.opensource.org/licenses/gpl-license.php GNU/GPL v.2 .
 */

global $DEFAULT_MOSEASYMEDIA_FIND_REGEXP;
global $DEFAULT_MOSEASYMEDIA_PARSE_ATTRIB_REGEXP;
global $DEFAULT_MOSEASYMEDIA_PARSE_REGEXP;


global $VALID_OVERRIDE_ATTRIBUTES;
$VALID_OVERRIDE_ATTRIBUTES = array("media",
"width", "height", "displayheight",
"autostart", "autoplay", "play", "autoload",
"loop", "playcount", "repeat", "repeat_playlist", "radio_mode",
"autosize", "overstretch", "stretchtofit",
"player",
"logo", "previewimage", "previewmessage", "image", "title", "link",
"volume",
"controls", "mode", "largecontrols",
"custommode", "wmode", "bgcolor", "shuffle",
"popup", "popupwidth", "popupheight", "popuptoolbar", "popupmenubar", "popupscrollbars", "popupmediathumbnail"
);

global $MANDATORY_ATTRIBUTES;
$MANDATORY_ATTRIBUTES = array("media");

global $JSANDCSS;
$JSANDCSS = array();


/**
 * <p>moseasymediaBot class handles the parsing the mambot out of the content and embedding the HTMLized output back in the content.</p>
 * <p>HTMLization takes place in moseasymediaMedia class.</p>
 * @see moseasymediaMedia::moseasymediaMedia()
 */
class moseasymediaBot {

	/**
	 * Function: This is where it all starts. Our mambot's content is getting prepared, right here, right now...
	 * @param bool $published
	 * @param string $text
	 * @param object $moseasymediaparams
	 * @return   either rendered media embed code, or the text that's passed
	 */
	function render_moseasymedia( $text, $moseasymediaparams) {

		/**
		 * Good and bad {moseasymedia} tags.
		 * {moSeasyMedia media=www.blah.com/movie.mpeg}   GOOD
		 * {moSeasyMedia media:www.blah.com/movie.mpeg}   BAD
		 * {moseasymedia media=  }  BAD
		 * {moseasymedia =dfdsfsd}  BAD
		 * {moseasymedia =/videos/tripvideos/falling.swf}   BAD
		 * {moseasymedia media   =      /videos/tripvideos/falling.swf}   GOOD
		 * {moseasymedia      media                 = ./images/stories/somequicktimemovie.mov GOOD}
		 * {moseasyMedia media=www.blah.com/movie.mpeg height=400}   GOOD
		 * {moSeasyMedia media=www.blah.com/movie.mpeg width=300}   GOOD
		 * {moSeaYyMedia media=www.blah.com/movie.mpeg height=400 width=300}   GOOD
		 */

		global $DEFAULT_MOSEASYMEDIA_FIND_REGEXP;
		if ( preg_match_all( $DEFAULT_MOSEASYMEDIA_FIND_REGEXP, $text, $moseasymediabots, PREG_SET_ORDER ) ) {
			//sample bots array after preg_match_all
			//Array ( [0] => Array ( [0] => {moseasymedia               media=http://www.brilaps.com/video1.mpg} )
			//                  [1] => Array ( [0] => {moseasymedia media=http://www.brilaps.com/video2.mpg    } )
			//                  [2] => Array ( [0] => {moseasymedia media=http://www.brilaps.com/video3.mpg width=100  height=120} ) )
			//These are to be processed in the insertBots and following functions.
			return moseasymediabot::insertBots($text, $moseasymediabots, $moseasymediaparams);
		}
		else
		return $text;
	}

	/**
	 * Function: Process each moseasymedia mambot entry in the content.
	 * (Process = make HTML out of them or HTMLization)
	 * @param string $textin Actual Content text
	 * @param array $bots mambot entries to be parsed and processed
	 * @param object $moseasymediaparams  parameters
	 * @return string Final content string
	 */
	function insertBots( $textin, $moseasymediabots, $moseasymediaparams ) {

		global $DEFAULT_MOSEASYMEDIA_FIND_REGEXP, $DEFAULT_MOSEASYMEDIA_PARSE_REGEXP,$DEFAULT_MOSEASYMEDIA_PARSE_ATTRIB_REGEXP, $MOSEASYMEDIA_BASE_URL, $MOSEASYMEDIA_ABSOLUTE_PATH;
		global $JSANDCSS;
		
		// split the content text with the same regular expression.
		$text = preg_split($DEFAULT_MOSEASYMEDIA_FIND_REGEXP, $textin );
		$textout = $text[0];


		//process the bots
		for($i = 0; $i <= count($moseasymediabots) - 1; $i++) {

			require_once($MOSEASYMEDIA_ABSOLUTE_PATH. '/moseasymediaParams.php');
            // params per media about to get handled. we don't want to carry over the possibly updated one from the previous call  
            // so we create a new instance          
			$moseasymediaparamspm = new moseasymediaParams();
			$moseasymediaparamspm->setParams($moseasymediaparams->getParams());

			// we need to re-parse each mambot entry.

			//&nbsp; is evil. We gotta "normalize"(kinda) the string.
			//in case, some html string gets stuck in
			$moseasymediastring = strip_tags($moseasymediabots[$i][0]);
			$pattern = "/(\s|\&nbsp;)+/";
			$replace = " ";
			$moseasymediastringN = preg_replace($pattern, $replace, $moseasymediastring);

			//get rid of all the unnecessary whitespace. This is odd but I felt like having a cleaner bot string
			$pattern = "/((\{)\s*)|(\s*(=)\s*)|(\s*(\}))/";
			$replace = "\$2\$4\$6";
			$moseasymediastringN = preg_replace($pattern, $replace, $moseasymediastringN);

			//and finally get rid of { and } ..
			$moseasymediastringN = substr($moseasymediastringN, 1, strlen($moseasymediastringN) - 2);

			//parse the bot attributes into an array
			if ( preg_match_all( "/".$DEFAULT_MOSEASYMEDIA_PARSE_ATTRIB_REGEXP."{1}/", $moseasymediastringN, $attribs, PREG_SET_ORDER ) ) {
				//Sample output of regex parse
				//Array ( [0] => Array ( [0] => media=http://www.brilaps.com/video3.mpg [1] => media=http://www.brilaps.com/video3.mpg [2] => media [3] => http://www.brilaps.com/video3.mpg )
				//         [1] => Array ( [0] => width=100 [1] => width=100 [2] => width [3] => 100 )
				//         [2] => Array ( [0] => height=120 [1] => height=120 [2] => height [3] => 120 )
				//         [3] => Array ( [0] => blah=12 [1] => blah=12 [2] => blah [3] => 12 )
				//         [4] => Array ( [0] => wlah=rewrew} [1] => wlah=rewrew} [2] => wlah [3] => rewrew) )
				//**blah and wlah sections are to show that additional attributes can also be handled if necessary

				//start embedding the media according to the attrib array above. (bits and pieces of $moseasymediaparams may be overridden)				
				$textout .= moseasymediabot::processBot($attribs, $moseasymediaparamspm);
				//skip the {moseasymedia .....} strings. The line above already HTMLized that string
				$textout .= $text[$i+1];
				unset($moseasymediaparamspm);
			}
			unset($attribs);
		}		
		return $textout;
	}

	/**
	 * Function: Process the individual moseasymedia mambot. Check validity, load mambot params etc...
	 * @param $attribs atribute array for the mambot
	 * @param $moseasymediaparams 
	 * @return string HTMLized mambot.
	 */
	function processBot( $attribs, $moseasymediaparams ) {

		global $VALID_OVERRIDE_ATTRIBUTES, $MANDATORY_ATTRIBUTES;;

		//with all the regex parsing above, we can be certain that [n][2] will be the attrib name, [n][3] will be attib value
		// [n][0] and [n][1] are the same but regex stuff already gave me enough trouble, so I will ignore that oddity

		//make a nice and clean attrib array.
		//Array ( [media] => http://www.brilaps.com/video3.mpg [width] => 100 [height] => 120 [blah] => 12 [wlah] => rewrew )

		$bValidMedia = FALSE;

		$mediaAttribs = array();
		foreach ($attribs as $attrib) {
			if(is_array($attrib)){
				if (in_array(strtolower($attrib[2]), $VALID_OVERRIDE_ATTRIBUTES))   {
					if (in_array(strtolower($attrib[2]), $MANDATORY_ATTRIBUTES)) {
						//just so that we won't deal with media, url, link, file checks from now on. we know that this is the file location now.
						//trim the quotes and whitespace.
						$mediaAttribs["media"] = strip_tags (trim($attrib[3], "\x22\x27"));
						$bValidMedia = TRUE;
					}
					else
					$mediaAttribs[$attrib[2]] = trim($attrib[3]);
				}
			}
		}

		if (!$bValidMedia) {
			$text = "<p>Incorrect format. Please use { moseasymedia media=path_to_your_file .....} format to properly render your media";
			$text .= "<br/><a href='http://forum.brilaps.com/' target='null'>support forum of brilaps.com</a></p>";
			return $text;
		}
		else {
			global $MOSEASYMEDIA_ABSOLUTE_PATH;
			/**
			 * require moseasymediaPluginEngine.class.php for actual embedding.
			 */
			require_once( $MOSEASYMEDIA_ABSOLUTE_PATH.'moseasymediaPluginEngine.class.php');
            foreach ($mediaAttribs as $k => $v)
                $mediaAttribs[$k] = trim($v, "\x22\x27");

			$mediaPluginEngine = new moseasymediaPlayerEngine($mediaAttribs, $moseasymediaparams);
			$text = $mediaPluginEngine->embed( );
			$text .= "<noscript><div class='moseasymedia-noscript'>".$moseasymediaparams->get("noscript", "Please turn on javascript to view the media.")."</div></noscript>";
			return $text;
		}
	}
}
?>
